import threading
import sys
from getch import getch

CONTROL_C = b"\x03"


class KeyboardReadThread(threading.Thread):
    def __init__(self, terminate_queue, net_socket):
        threading.Thread.__init__(self)
        self.terminate_queue = terminate_queue
        self.net_socket = net_socket

    def run(self):

        while True:
            read_key = getch()

            if read_key == b"\r":
                read_key = b"\r\n"

            if read_key == CONTROL_C:
                break

            self.sendall(read_key)
        self.terminate_queue.put(None)

    def sendall(self, content):
        sys.stdout.buffer.write(content)
        sys.stdout.flush()
        """ try to send to the remote end """
        try:
            self.net_socket.sendall(content)
        except Exception as ex:
            print(ex, file=sys.stderr)
            self.terminate_queue.put(None)



import threading
import sys
import socket

BUFFER_SIZE = 4096


class NetworkReadThread(threading.Thread):
    def __init__(self, terminate_queue, net_socket):
        threading.Thread.__init__(self)
        self.terminate_queue = terminate_queue
        self.net_socket = net_socket

    def run(self):
        while True:
            try:
                data = self.net_socket.recv(BUFFER_SIZE)
            except socket.timeout:
                data = ""
            if len(data) == 0:  # Connection was closed
                break
            sys.stdout.buffer.write(data)
            sys.stdout.flush()
        self.terminate_queue.put(None)



import socket
from queue import Queue
from .network import NetworkReadThread
from .keyboard import KeyboardReadThread


class NetCatClient:
    def __init__(self, hostname, port):
        self.hostname = hostname
        self.port = port

    def connect(self, timeout=10.0):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(timeout)
        s.connect((self.hostname, int(self.port)))
        s.settimeout(None)
        self.net_socket = s

    def interactive_shell(self):
        terminate_queue = Queue()

        net_thread = NetworkReadThread(terminate_queue, self.net_socket)
        net_thread.setDaemon(True)
        net_thread.start()

        kbd_thread = KeyboardReadThread(terminate_queue, self.net_socket)
        kbd_thread.setDaemon(True)
        kbd_thread.start()

        _ = terminate_queue.get()



import argparse
from .netcat import NetCatClient


class CommandArgs:
    def __init__(self):
        parser = argparse.ArgumentParser(description="Process some integers.")
        parser.add_argument("-v", help="Set verbose mode", action="store_true")
        parser.add_argument("hostname", help="Connect to hostname", metavar="HOSTNAME")
        parser.add_argument("port", help="Using port", metavar="PORT")
        parser.add_argument(
            "-t", help="Max connection timeout (secs)", metavar="TIMEOUT"
        )
        self.parser = parser

    def parse(self):
        self.args = self.parser.parse_args()

    def run(self):
        if self.args.hostname:
            if self.args.v:
                print(f"* Connecting to {self.args.hostname}, port {self.args.port}")
            netcat = NetCatClient(self.args.hostname, self.args.port)
            netcat.connect()
            if self.args.v:
                print("* Connected")
            netcat.interactive_shell()
